local mod = ...



-- Building Asset Processor
mod:registerAssetProcessor("models/Camera1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Asset
mod:registerAssetId("models/Camera1.fbx/Prefab/Camera.Part", "PREFAB_BIRDSEYE_CAMERA")
mod:registerAssetId("models/Camera1.fbx/Prefab/Bird.Part", "PREFAB_BIRDSEYE_BIRD")



-- Building -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_BIRDSEYE",
	Name = "BUILDING_BIRDSEYE_NAME",
	Description = "BUILDING_BIRDSEYE_DESC",
    BuildingType = "MODS",
	AssetCoreBuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA",
    AssetMiniatureBuildingPart = "BUILDING_PART_BIRDSEYE_BIRD",
})



-- Component -------------------------------------------------------------------------------------
local COMP_BIRDSEYE_CAMERA = {
	TypeName = "COMP_BIRDSEYE_CAMERA",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "Init", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "Height", Type = "integer", Default = 500, Flags = { "SAVE_GAME" } },
    }
}

-- Init -------------------------------------------------------------------------------------
function COMP_BIRDSEYE_CAMERA:update()

    if not self.Init then

        -- Raise the parent object to the required height
        local obj = self:getOwner()
        obj:translate({0, self.Height, 0})

        -- Don't repeat this later
        self.Init = true

    end

end

-- Register custom Class
mod:registerClass(COMP_BIRDSEYE_CAMERA)



-- Building Part: Bird
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_BIRDSEYE_BIRD",
    Name = "BUILDING_PART_BIRDSEYE_CAMERA_NAME",
    IsVisibleWhenBuilt = false,
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_BIRDSEYE_BIRD",
        MiniatureConfig = {
            ClipUnderGround = false,
            GroupHeight = 0.0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 15.0,
            CameraPitchOffset = 35.0
        },
    },
})



-- Register Camera -------------------------------------------------------------------------------------
local function registerCamera(height)

    local prefab = "PREFAB_BIRDSEYE_CAMERA" .. height
    local part = "BUILDING_PART_BIRDSEYE_CAMERA" .. height

    -- Asset
    mod:registerAssetId("models/Camera1.fbx/Prefab/Camera." .. height, prefab)

    -- Attach class to prefab
    mod:registerPrefabComponent(prefab, { DataType = "COMP_BIRDSEYE_CAMERA", Height = height } )

    -- Building Part
    mod:registerAsset({
        DataType = "BUILDING_PART",
        Id = part,
        Name = part .. "_NAME",
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = prefab,
        },
        AssetBuildingFunction = "BUILDING_FUNCTION_BIRDSEYE_CAMERA" .. height,
    })

    -- Building Function
    mod:registerAsset({
        DataType = "BUILDING_FUNCTION_WORKPLACE",
        Id = "BUILDING_FUNCTION_BIRDSEYE_CAMERA" .. height,
        Name = part .. "_NAME",
        IsPrivate = true,
        IsPausable = false,
        HasResourceDepot = false,
        WorkerCapacity = 0,
        RelatedJob = {
            Job = "BUILDER",
            Behavior = "BEHAVIOR_WORK"
        },
        InputInventoryCapacity = {
        },
        ResourceListNeeded = {
        },
        ResourceProduced = {
        }
    })

end

registerCamera(50)
registerCamera(250)
registerCamera(500)
registerCamera(750)
registerCamera(1000)
registerCamera(1250)



-- Assemblage -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_BIRDSEYE_CAMERA_ASSEMBLAGE",
    Name = "BUILDING_PART_BIRDSEYE_CAMERA_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
        CoreRandomBuildingPartList = { "PREFAB_BIRDSEYE_CAMERA" },
        MandatoryBuildingPartList = {
            { BuildingPart = "BUILDING_PART_BIRDSEYE_BIRD" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA50" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA250" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA500" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA750" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA1000" },
            { BuildingPart = "BUILDING_PART_BIRDSEYE_CAMERA1250" },
        },
    },
})

mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_BIRDSEYE_CAMERA",
    Name = "BUILDING_PART_BIRDSEYE_CAMERA_NAME",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_BIRDSEYE_CAMERA_ASSEMBLAGE"
        },
    },
    IsVisibleWhenBuilt = false,
})





